"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
// import { Field, ObjectType } from 'type-graphql';
const typeorm_1 = require("typeorm");
const utils_1 = require("../utils");
// import { ConnectionType, EdgeType } from '../sgql';
// import { getMetadataStorage } from 'type-graphql/dist/metadata/getMetadataStorage';
const gluegun_1 = require("gluegun");
function OneToOne(parentType, joinFunc, options = {}) {
    const nullable = 'nullable' in options ? options.nullable : true;
    const factories = [
        /*((target: any, propertyKey: string, descriptor: PropertyDescriptor): any => {
          Field(parentType, { nullable, ...options, name: strings.camelCase(propertyKey) })(
            target,
            propertyKey,
            descriptor
          );
        }) as MethodDecoratorFactory,*/
        ((target, propertyKey, descriptor) => {
            typeorm_1.OneToOne(parentType, joinFunc, {
                cascade: true,
                onDelete: nullable ? 'SET NULL' : 'NO ACTION',
                ...options,
                name: gluegun_1.strings.camelCase(propertyKey)
            })(target, propertyKey, descriptor);
        }),
        ((target, propertyKey, descriptor) => {
            typeorm_1.JoinColumn({ ...options, name: gluegun_1.strings.camelCase(propertyKey) })(target, propertyKey, descriptor);
        })
    ];
    return utils_1.composeMethodDecorators(...factories);
}
exports.OneToOne = OneToOne;
function ManyToOne(parentType, joinFunc, options = {}) {
    const nullable = 'nullable' in options ? options.nullable : true;
    const factories = [
        /*((target: any, propertyKey: string, descriptor: PropertyDescriptor): any => {
          Field(parentType, { nullable, ...options, name: strings.camelCase(propertyKey) })(
            target,
            propertyKey,
            descriptor
          );
        }) as MethodDecoratorFactory,*/
        ((target, propertyKey, descriptor) => {
            typeorm_1.ManyToOne(parentType, joinFunc, {
                cascade: ['insert', 'update'],
                onDelete: nullable ? 'SET NULL' : 'NO ACTION',
                ...options,
                name: gluegun_1.strings.camelCase(propertyKey)
            })(target, propertyKey, descriptor);
        }),
        ((target, propertyKey, descriptor) => {
            typeorm_1.JoinColumn({ ...options, name: gluegun_1.strings.camelCase(propertyKey) })(target, propertyKey, descriptor);
        })
    ];
    return utils_1.composeMethodDecorators(...factories);
}
exports.ManyToOne = ManyToOne;
function OneToMany(parentType, joinFunc, options = {}) {
    const nullable = 'nullable' in options ? options.nullable : false;
    const factories = [
        ((target, propertyKey, descriptor) => {
            typeorm_1.OneToMany(parentType, joinFunc, {
                cascade: ['insert', 'update'],
                onDelete: nullable ? 'SET NULL' : 'NO ACTION',
                ...options,
                name: gluegun_1.strings.camelCase(propertyKey)
            })(target, propertyKey, descriptor);
        })
        /*(target: any, propertyKey: string, descriptor: PropertyDescriptor): any => {
          setTimeout(() => {
            const parentResult = parentType();
            if (!parentResult) return;
            const parentResultName = strings.pascalCase(parentResult.name);
            const objectEdge =
              (
                getMetadataStorage().objectTypes.find(
                  object => object.name === `${parentResultName}EdgeRelay`
                ) || ({} as any)
              ).target || class extends EdgeType(parentResult) {};
            const objectConnection =
              (
                getMetadataStorage().objectTypes.find(
                  object => object.name === `${parentResultName}ConnectionRelay`
                ) || ({} as any)
              ).target || class extends ConnectionType(parentResult, objectEdge) {};
            if (objectEdge.name !== `${parentResultName}EdgeRelay`) {
              Object.defineProperty(objectEdge, 'name', { value: `${parentResultName}EdgeRelay` });
              ObjectType()(objectEdge);
            }
            if (objectConnection.name !== `${parentResultName}ConnectionRelay`) {
              Object.defineProperty(objectConnection, 'name', {
                value: `${parentResultName}ConnectionRelay`
              });
              ObjectType()(objectConnection);
            }
            Reflect.defineMetadata(
              'design:type:backup',
              Reflect.getMetadata('design:type', target, propertyKey),
              target,
              propertyKey
            );
            Reflect.defineMetadata('design:type', undefined, target, propertyKey);
            Field(() => objectConnection, { nullable, ...options })(target, propertyKey, descriptor);
            Reflect.defineMetadata(
              'design:type',
              Reflect.getMetadata('design:type:backup', target, propertyKey),
              target,
              propertyKey
            );
          });
        }*/
    ];
    return utils_1.composeMethodDecorators(...factories);
}
exports.OneToMany = OneToMany;
function ManyToMany(parentType, joinFunc, options = {}) {
    // const nullable = 'nullable' in options ? options.nullable : false;
    const factories = [
        typeorm_1.ManyToMany(parentType, joinFunc, {
            cascade: ['insert', 'update'],
            // onDelete:'CASCADE',
            ...options
        }),
        ((target, propertyKey, descriptor) => {
            const parentResult = parentType();
            if (!parentResult)
                return;
            const returnType = parentResult.prototype.constructor.name;
            const [A, B] = [`${target.constructor.name}_${propertyKey}`, returnType].sort();
            typeorm_1.JoinTable({
                name: `_${gluegun_1.strings.pascalCase(A)}To${gluegun_1.strings.pascalCase(B)}`,
                joinColumn: {
                    name: gluegun_1.strings.camelCase(target.constructor.name)
                },
                inverseJoinColumn: {
                    name: gluegun_1.strings.camelCase(returnType)
                }
            })(target, propertyKey, descriptor);
        })
        /*(target: any, propertyKey: string, descriptor: PropertyDescriptor): any => {
          setTimeout(() => {
            const parentResult = parentType();
            if (!parentResult) return;
            const parentResultName = strings.pascalCase(parentResult.name);
            const objectEdge =
              (
                getMetadataStorage().objectTypes.find(
                  object => object.name === `${parentResultName}EdgeRelay`
                ) || ({} as any)
              ).target || class extends EdgeType(parentResult) {};
            const objectConnection =
              (
                getMetadataStorage().objectTypes.find(
                  object => object.name === `${parentResultName}ConnectionRelay`
                ) || ({} as any)
              ).target || class extends ConnectionType(parentResult, objectEdge) {};
            if (objectEdge.name !== `${parentResultName}EdgeRelay`) {
              Object.defineProperty(objectEdge, 'name', { value: `${parentResultName}EdgeRelay` });
              ObjectType()(objectEdge);
            }
            if (objectConnection.name !== `${parentResultName}ConnectionRelay`) {
              Object.defineProperty(objectConnection, 'name', {
                value: `${parentResultName}ConnectionRelay`
              });
              ObjectType()(objectConnection);
            }
            Reflect.defineMetadata(
              'design:type:backup',
              Reflect.getMetadata('design:type', target, propertyKey),
              target,
              propertyKey
            );
            Reflect.defineMetadata('design:type', undefined, target, propertyKey);
            Field(() => objectConnection, {
              nullable,
              ...options,
              name: strings.camelCase(propertyKey)
            })(target, propertyKey, descriptor);
            Reflect.defineMetadata(
              'design:type',
              Reflect.getMetadata('design:type:backup', target, propertyKey),
              target,
              propertyKey
            );
          });
        }*/
    ];
    return utils_1.composeMethodDecorators(...factories);
}
exports.ManyToMany = ManyToMany;
