"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const prettier_1 = __importDefault(require("prettier"));
const typedi_1 = require("typedi");
const logger_1 = require("../core/logger");
const TypeORMConverter_1 = require("./TypeORMConverter");
class SchemaGenerator {
    static generate(entities, tcgImportPath = 'tcg') {
        let template = `
      // This file has been auto-generated by Tcg. Do not update directly as it will be re-written.
      import 'reflect-metadata';
      import { ArgsType, Field as FieldGraphQL, Float, GraphQLISODateTime,
               ID, InputType,
               ObjectType, registerEnumType } from 'type-graphql';

      import { GraphQLBoolean, GraphQLFloat, GraphQLID,
               GraphQLInt, GraphQLString } from 'graphql';
      export { GraphQLBoolean, GraphQLFloat, GraphQLID,
               GraphQLInt, GraphQLString };

      import { GraphQLJSONObject }  from 'graphql-type-json';
      export { GraphQLJSONObject };

      import { IDType, ConnectionArgs, PageInfo } from '${tcgImportPath}';
      export { IDType, ConnectionArgs, PageInfo };
      ${TypeORMConverter_1.entityListToImports(entities)}
      ${TypeORMConverter_1.entityToCreateInputsImporter(entities)}
      ${TypeORMConverter_1.entityToUpdateInputsImporter(entities)}
    `;
        entities
            .filter((entity) => entity.tableType !== 'junction')
            .forEach((entity) => {
            template += `
        ${TypeORMConverter_1.entityToOrderByEnum(entity)}
        ${TypeORMConverter_1.entityToConnectionResponse(entity)}
        ${TypeORMConverter_1.entityToCreateInputImporter(entity)}
        ${TypeORMConverter_1.entityToUpdateInputImporter(entity)}
        ${TypeORMConverter_1.entityToUpdateInputDataImporter(entity)}
        ${TypeORMConverter_1.entityToUpdateManyWithWhereNestedInputImporter(entity)}
        ${TypeORMConverter_1.entityToWhereInputImporter(entity)}
        ${TypeORMConverter_1.entityToWhereScalarInputImporter(entity)}
        ${TypeORMConverter_1.entityToWhereUniqueInput(entity)}
        ${TypeORMConverter_1.entityToWhereArgs(entity)}
        ${TypeORMConverter_1.entityToCreateManyArgs(entity)}
        ${TypeORMConverter_1.entityToUpdateInputArgs(entity)}
      `;
        });
        return this.format(template);
    }
    static format(code, options = {}) {
        try {
            return prettier_1.default.format(code, {
                ...options,
                parser: 'typescript'
            });
        }
        catch (e) {
            this.logger.log(`There is a syntax error in generated code, unformatted code printed, error: ${JSON.stringify(e)}`);
            return code;
        }
    }
}
exports.SchemaGenerator = SchemaGenerator;
SchemaGenerator.logger = typedi_1.Container.has('tcg.logger') ? typedi_1.Container.get('tcg.logger') : logger_1.logger;
