"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typeorm_1 = require("typeorm");
const utils_1 = require("../utils");
exports.createDBConnection = (dbOptions = {}) => {
    const config = {
        ...getBaseConfig(),
        ...dbOptions
    };
    if (!config.database) {
        throw new Error(`createConnection: 'database' is required`);
    }
    return typeorm_1.createConnection(config);
};
exports.mockDBConnection = (dbOptions = {}) => {
    return exports.createDBConnection({
        ...dbOptions,
        database: 'tcg.sqlite.tmp',
        synchronize: true,
        type: 'sqlite'
    });
};
function getBaseConfig() {
    return {
        cli: {
            entitiesDir: 'src/models',
            migrationsDir: 'src/migration',
            subscribersDir: 'src/subscriber'
        },
        database: utils_1.getDatabaseName(),
        entities: utils_1.getDatabaseEntityPaths(),
        host: utils_1.getDatabaseHost(),
        logger: 'advanced-console',
        logging: utils_1.getDatabaseLoggingLevel(),
        migrations: utils_1.getDatabaseMigrationPaths(),
        password: utils_1.getDatabasePassword(),
        port: utils_1.getDatabasePort(),
        subscribers: utils_1.getDatabaseSubscriberPaths(),
        synchronize: utils_1.shouldSchronizeDatabaseSchema(),
        type: utils_1.getDatabaseType(),
        username: utils_1.getDatabaseUsername()
    };
}
exports.getBaseConfig = getBaseConfig;
