/// <reference types="node" />
import { ApolloServer, OptionsJson } from 'apollo-server-express';
import { Request } from 'express';
import { GraphQLSchema } from 'graphql';
import { Binding } from 'graphql-binding';
import { Server as HttpServer } from 'http';
import { Server as HttpsServer } from 'https';
import { AuthChecker } from 'type-graphql';
import { Container } from 'typedi';
import { Connection, ConnectionOptions } from 'typeorm';
import { Logger } from './logger';
import { BaseContext } from './Context';
export interface ServerOptions<T> {
    container?: Container;
    authChecker?: AuthChecker<T>;
    autoGenerateFiles?: boolean;
    context?: (request: Request) => object;
    protocol?: 'http' | 'https';
    host?: string;
    port?: string | number;
    path?: string;
    token?: string;
    generatedFolder?: string;
    logger?: Logger;
    middlewares?: any[];
    mockDBConnection?: boolean;
    openPlayground?: boolean;
    resolversPath?: string[];
    tcgImportPath?: string;
    introspection?: boolean;
    bodyParserConfig?: OptionsJson;
}
export declare class Server<C extends BaseContext> {
    private appOptions;
    private dbOptions;
    appProtocol: 'http' | 'https';
    appHost: string;
    appPort: number;
    appPath: string;
    appToken: string;
    authChecker: AuthChecker<C>;
    autoGenerateFiles: boolean;
    connection: Connection;
    container: Container;
    generatedFolder: string;
    graphQLServer: ApolloServer;
    httpServer: HttpServer | HttpsServer;
    logger: Logger;
    mockDBConnection: boolean;
    resolversPath: string[];
    schema?: GraphQLSchema;
    introspection: boolean;
    bodyParserConfig?: OptionsJson;
    constructor(appOptions: ServerOptions<C>, dbOptions?: Partial<ConnectionOptions>);
    getLogger(): Logger;
    establishDBConnection(): Promise<Connection>;
    getBinding(options?: {
        origin?: string;
        token?: string;
    }): Promise<Binding>;
    buildGraphQLSchema(): Promise<GraphQLSchema>;
    generateFiles(): Promise<void>;
    start(): Promise<this>;
    stop(): Promise<void>;
    private shouldOpenPlayground;
}
